/*  Copyright 2009 Marc Toussaint
    email: mtoussai@cs.tu-berlin.de

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a COPYING file of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/> */

#ifndef MT_qhull_h
#define MT_qhull_h

#include "array.h"
#include "ors.h"

void plotQhullState(uint D);

extern int QHULL_DEBUG_LEVEL;

double distanceToConvexHull(const arr &X,        //points
			    const arr &y,        //query point
			    arr *projectedPoint, //return query point projected on closest facet
			    uintA *faceVertices, //return indices of vertices of closest facet
			    bool freeqhull);     //free allocated qhull engine after request [true]

double distanceToConvexHullGradient(arr& dDdX,   //gradient (or same dim as X)
				const arr &X,    //points
			    const arr &y,        //query point
			    arr *projectedPoint, //query point projected on closest facet
			    uintA *faceVertices, //indices of vertices of closest facet
			    bool freeqhull);     //free allocated qhull engine after request [true]

double forceClosure(const arr& X,  //contact points (size Nx3)
		    const arr& Xn, //contact normals (size Nx3)
		    const ors::Vector& center, //object center
		    float mu,     //friction coefficient
		    float discountTorques,     //friction coefficient
		    arr *dFdX);    //optional: also compute gradient

double forceClosureFromProxies(ors::Graph& C,uint i);

void getTriangulatedHull(uintA& T, arr& V);

void getDelaunayEdges(uintA& E, const arr& V);
                         
//OLD CODE specifically for graphs...
//template<class N,class E>
//void delaunay(Graph<N,E>& g,uint dim=2){

#ifdef MT_IMPLEMENTATION
#  include "ors_qhull.cpp"
#endif

#endif
