/*  Copyright 2009 Marc Toussaint
    email: mtoussai@cs.tu-berlin.de

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a COPYING file of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/> */

#ifndef MT_plot_h
#define MT_plot_h

//===========================================================================

namespace MT{
  template<class T> class Array;
}
typedef MT::Array<double> arr;

struct Gaussian;
typedef MT::Array<Gaussian> gaussA;
typedef MT::Array<Gaussian*> gaussPA;

class OpenGL;

//===========================================================================

typedef enum{ opengl, xfig, gnupl } PlotMode;

struct PlotData;
struct PlotModule{
  PlotMode mode;
  OpenGL *gl;
  bool light,grid,colors,drawBox,drawDots;	       //display options
  PlotModule();
  ~PlotModule();
};
extern PlotModule plotModule;

//===========================================================================

void plotGnuplot();
void plotOpengl();
void plotOpengl(bool threeD,double xl,double xh,double yl=-1.,double yh=1.,double zl=-1.,double zh=1.);

void plot(bool wait=true);
void plotClear();
void plotFunction(const arr& f,double x0=0.,double x1=0.);
void plotFunctions(const arr& f,double x0=0.,double x1=0.);
void plotFunction(const arr& x,const arr& f);
void plotSurface(const arr& X);
void plotArray(const arr& X);
void plotPoint(double x,double y,double z);
void plotPoints(const arr& X);
void plotClearPoints();
void plotLine(const arr& X);
void plotPoints(const arr& X,const arr& Y);
void writeGnuplotFiles();
void plotCovariance(const arr& mean,const arr& cov);
void plotVectorField(const arr& X,const arr& dX);
void plotVectorField(arr& dX);
void plotGaussians(gaussA& G);
void plotGaussians(gaussPA& G);

void plotDrawOpenGL(void *data);

#ifdef MT_IMPLEMENTATION
#  include"plot.cpp"
#endif

#endif

